@@ -21,6 +21,11 @@ class ListManager(object):
 
     This class is not meant to be called itself, but rather for its
     methods to be called by the service object.
+
+    Note that the methods for creating lists can conflict in threaded applications, if 
+    two threads are each allocated the same unused list name. You are
+    strongly advised to use locks to synchronise any list creation requests (create_list, 
+    or intersect, union, subtract, diff) unless you are choosing your own names each time.
     """
 
     DEFAULT_LIST_NAME = "my_list_"
@@ -88,7 +93,16 @@ class ListManager(object):
         return len(self.get_all_list_names())
 
     def get_unused_list_name(self):
-        """Get a list name that does not conflict with any lists you have access to"""
+        """
+        Get an unused list name 
+        =======================
+
+        This method returns a new name that does not conflict
+        with any currently existing list name. 
+
+        The list name is only guaranteed to be unused at the time
+        of allocation. 
+        """
         list_names = self.get_all_list_names()
         counter = 1
         name = self.DEFAULT_LIST_NAME + str(counter)
@@ -98,6 +112,28 @@ class ListManager(object):
         self._temp_lists.add(name)
         return name
 
+
+    def _create_list_from_queryable(self, queryable, name, description, tags):
+        q = queryable.to_query()
+        if not q.views:
+            q.add_view(q.root.name + ".id")
+        else: 
+            # Check to see if the class of the selected items is unambiguous
+            up_to_attrs = set((v[0:v.rindex(".")] for v in q.views))
+            if len(up_to_attrs) == 1:
+                q.select(up_to_attrs.pop() + ".id")
+
+        uri = q.get_list_upload_uri()
+        params = q.to_query_params()
+        params["listName"] = name
+        params["description"] = description
+        params["tags"] = ";".join(tags)
+        form = urllib.urlencode(params)
+        resp = self.service.opener.open(uri, form)
+        data = resp.read()
+        resp.close()
+        return self.parse_list_upload_response(data) 
+
     def create_list(self, content, list_type="", name=None, description=None, tags=[]):
         """
         Create a new list in the webservice
@@ -107,6 +143,9 @@ class ListManager(object):
         list, and will be automatically deleted when the program ends. To prevent
         this happening, give the list a name, either on creation, or by renaming it.
 
+        This method is not thread safe for anonymous lists - it will need synchronisation
+        with locks if you intend to create lists with multiple threads in parallel.
+
         @rtype: intermine.lists.List
         """
         if description is None:
@@ -122,24 +161,17 @@ class ListManager(object):
                 ids = content
             else:
                 try:
+                    return self._create_list_from_queryable(content, name, description, tags)
+                except AttributeError:
                     ids = "\n".join(map(lambda x: '"' + x + '"', iter(content)))
-                except TypeError:
-                    try:
-                        uri = content.get_list_upload_uri()
-                    except:
-                        content = content.to_query()
-                        uri = content.get_list_upload_uri()
-                    params = content.to_query_params()
-                    params["listName"] = name
-                    params["description"] = description
-                    form = urllib.urlencode(params)
-                    resp = self.service.opener.open(uri, form)
-                    data = resp.read()
-                    resp.close()
-                    return self.parse_list_upload_response(data) 
 
         uri = self.service.root + self.service.LIST_CREATION_PATH
-        query_form = {'name': name, 'type': list_type, 'description': description, 'tags': ";".join(tags)}
+        query_form = {
+            'name': name, 
+            'type': list_type, 
+            'description': description, 
+            'tags': ";".join(tags)
+        }
         uri += "?" + urllib.urlencode(query_form)
         data = self.service.opener.post_plain_text(uri, ids)
         return self.parse_list_upload_response(data)
