@@ -115,7 +115,9 @@ class TestService(WebserviceTest): # pragma: no cover
 
     def testVersion(self):
         """The service should have a version"""
-        self.assertEqual(self.s.version, 100)
+        from numbers import Number
+        v = self.s.version
+        self.assertTrue(isinstance(v, Number))
 
     def testRelease(self):
         """The service should have a release"""
@@ -550,6 +552,7 @@ class TestQuery(WebserviceTest): # pragma: no cover
         self.q.set_logic("(A and B) or (A and C and D) and (E or F)")
         expected ='<query constraintLogic="((A and B) or (A and C and D)) and (E or F)" longDescription="" model="testmodel" name="" sortOrder="Employee.age asc" view="Employee.name Employee.age Employee.department.name"><join path="Employee.department" style="OUTER"/><constraint code="A" op="IS NOT NULL" path="Employee.name"/><constraint code="B" op="&gt;" path="Employee.age" value="10"/><constraint code="C" extraValue="Wernham-Hogg" op="LOOKUP" path="Employee.department" value="Sales"/><constraint code="D" op="ONE OF" path="Employee.department.employees.name"><value>John</value><value>Paul</value><value>Mary</value></constraint><constraint code="E" loopPath="Employee" op="=" path="Employee.department.manager"/><constraint code="F" op="IN" path="Employee" value="some list of employees"/><constraint path="Employee.department.employees" type="Manager"/></query>'        
         self.assertEqual(expected, self.q.to_xml())
+        self.assertEqual(expected, self.q.clone().to_xml()) # Clones must produce identical XML
 
     def testSugaryQueryConstruction(self):
         """Test use of operation coercion which is similar to SQLAlchemy"""
@@ -888,6 +891,10 @@ class TestQueryResults(WebserviceTest): # pragma: no cover
 
         self.do_unpredictable_test(logic)
 
+class MinimalResultsTest(TestQueryResults):
+
+    PATH = "/testservice/legacyjsonrows"
+
 class TestTSVResults(WebserviceTest): # pragma: no cover
 
     model = None
